#!/bin/bash
##
# Ask user to specify allowed IP address(es) for RoksNet Portal admin interface access.
# Find user SSH session remote IP and use it as default. 
# If that does not exist (e.g. when user has not logged in via SSH),
# use the IP already configured in SSH file.
# 
# Script takes an optional apache conf location argument, it should be run in root permissions.
# ./configure_admin_interface_ip.sh [Optional ssl.conf path]
##

##
# Validate configuration IP list. Checks for common errors, but not full validation.
#
# @param ip_conf string containing allowed IP-s.
# 	 The parameter format is IP v4 network addresses separated by spaces or commas.
#	 Term 'all' is also allowed.
#	 E.g '127.0.0.1 192.168.1.1 10.10.10.0/24'
# @return 0 if IP conf is considered valid, a non-zero integer if not
##
function is_valid_ip_conf {
	ip_conf=$1
	echo $ip_conf | grep -Eq "^(([0-9]{1,3}\.){3}[0-9]{1,3}(\/[0-9]{1,2})?|\s+|,)+|all$"
	return $?
}

# regex representing section that comes before allowed admin interface IP list in apache2 conf
admin_ip_bookmark_regex="<Location\s+\"\/[\/\*]*admin\/\*\">[^<]+Allow from "
# apache conf location
apache_conf_location=/etc/apache2/sites-available/ssl.conf
if [ "$1" != "" ]
then
	apache_conf_location=$1
fi

# if apache conf is not writable, return error
if [[ ! -w "$apache_conf_location" ]] 
then
   echo "configure_admin_interface_ip.sh: WARNING File $apache_conf_location cannot be accessed." \
	"Script should be run with root permissions and roksnet-base package installed."
   exit 1
fi

# find allowed IPs from apache2 conf by substituting the file content with IP list
apache_conf_admin_ip=$(perl -p -e \
	"BEGIN{undef $/;} s/.*$admin_ip_bookmark_regex([^\n]+).*|.*/\1/smg" \
	$apache_conf_location)

# Do not substitute IP if conf IP was empty, meaning conf file was not found
# or the specific admin IP list was not found
if [ "$apache_conf_admin_ip" == "" ]
then
	echo "configure_admin_interface_ip.sh: WARNING: Administrator interface allowed IPs were not found from conf file $apache_conf_location."
	exit 1
else
	echo "IPs currently allowed to access administrator interface are '$apache_conf_admin_ip' in $apache_conf_location."
	# find SSH session remote IP
	# IP address, if it exists, is read from netstat output
	ssh_remote_ip=$(netstat -tnpa | grep 'ESTABLISHED.*sshd' | 
		tail -n 1 | awk '{print $5}' | perl -nle 'print $& if m{.*(?=:)}')
	# if SSH remote IP exists, use that as default, else use existing IP in apache2 config
	if [ "$ssh_remote_ip" != "" ] && is_valid_ip_conf "$ssh_remote_ip"
	then
		default_admin_ip="$ssh_remote_ip"
		echo "User remote IP '$ssh_remote_ip'"
	else
		default_admin_ip="$apache_conf_admin_ip"
	fi
	
	new_admin_ip=""
	# Run user input loop
	while [ "$new_admin_ip" == "" ]
	do 
		echo -n "Please provide IP address(es) allowed to access administrator interface: [default: $default_admin_ip] "

		read new_admin_ip < /dev/tty
		if [ "$new_admin_ip" == "" ]
		then
			new_admin_ip="$default_admin_ip"
		fi
		# Check that input resembles list of IPv4 network addresses or keyward 'all'
		if ! (is_valid_ip_conf $new_admin_ip)
		then
			echo "Input '$new_admin_ip' is not a valid IPv4 network address."
			# set to empty so that current loop would continue
			new_admin_ip=""
		fi
	done

	if [ "$new_admin_ip" != "$apache_conf_admin_ip" ]
	then
		# Escape forward slashes in user input so it can be used as a perl-replacement string
		new_admin_ip_quoted="${new_admin_ip//[\/]/\\\/}"
		# Perform IP replacement in apache config file
		perl -pi -e "BEGIN{undef $/;} s/($admin_ip_bookmark_regex)[^\n]+/\${1}$new_admin_ip_quoted/smg" \
			$apache_conf_location
		service apache2 restart
		if [ $? -eq 0 ]
		then
			echo "Allowed administrator interface access from '$new_admin_ip' in $apache_conf_location."
		else
			echo "configure_admin_interface_ip.sh: WARNING: Allowed administrator interface access from '$new_admin_ip'"\
				"in $apache_conf_location, but failed to restart apache2."
		fi
	else
		echo "Administrator interface access '$new_admin_ip' was not changed."
		
	fi
	exit 0
fi
